Library ieee;
Use ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;

Entity ds1620_tr is
Port 	(	dclk	:	in	bit;							--	Input Clock
			en		:	in	integer range 0 to 7;			--	Input Select Function
			dqi		:	in	std_logic;						--	Input Series Data From DS1620
			dqo		:	out	bit;							--	Output Series Data From DS1620
			rst		:	out	bit;							--	Output Enable Chip DS1620
			zdq		:	out	bit;							--	Output Control Tri-state
			temp	:	out	std_logic_vector(8 downto 0)	--	Output Temperature (Parallel) Data
		);
End ;

Architecture ds1620_tr_a of ds1620_tr is
Begin
process (dclk)
Variable ptc : bit_vector(7 downto 0);
Variable ptc_w : bit_vector(15 downto 0);
Variable n : integer range 0 to 31 := 0;					--	Counter Sequence
Variable rn1 : integer range 0 to 31 := 0;					--	Range Data Out
Variable rn2 : integer range 0 to 31 := 0;					--	Range Data In
Variable nd : integer range 0 to 7 := 0;
Variable wnd : integer range 0 to 15 := 0;
Variable di : integer range 0 to 8 := 0;
Variable dti : integer range 0 to 8 := 0;
Begin



if dclk'event and dclk = '1' then

	if en = 0 then
		zdq <= '0';								--	DISABLE DQ_OUT TO SEND DATA
		rst <= '0';								--	DISABLE DS1620 CHIP
		n := 0;									--	RESET ALL PARAMETERS
		nd := 0;
		wnd := 0;
		di := 0;

	else
		if en = 1 then
			ptc_w	:=	"0000000000001100";		--	WRITE CONFIG 0Ch
			rn1 := 15;							--	RANGE SEND 15 BIT DATA
			rn2 := rn1;							--	RANGE RECEIVE 0 BIT DATA
	
		elsif en = 2 then
			ptc	:=	"11101110";					--	START CONVERSION EEh
			rn1 := 7;							--	RANGE SEND 15 BIT DATA
			rn2 := rn1;							--	RANGE RECEIVE 0 BIT DATA
		
		elsif en = 4 then
			ptc	:=	"10101010";					--	READ TEMPERATURE AAh
			rn1 := 7;							--	RANGE SEND 8 BIT DATA
			rn2 := 18;							--	RANGE RECEIVE 8 BIT DATA

		end if;

		if n > rn2 and en = 4 then
			zdq <= '0';							--	DISABLE DQ_OUT TO SEND DATA
			rst <= '0';							--	DISABLE DS1620 CHIP
--			dqo <= '0';					--	SEND DATA SET WRITE CONFIGURATION FROM REGISTOR TO DS1620

		elsif n > rn1 then						--	RECEIVE TEMPERATURE DATA
			zdq <= '0';
			if en = 4 then
				rst <= '1';		
				di := di + 1;				
				dti := di - 2;
				temp(dti) <= dqi;				--	Convert Series Data to Parallel Data
			else
				rst <= '0';
			end if;
			n := n + 1;							--	NEXT CLOCK

		else									--	SEND PORTOCOL OR 
			zdq <= '1';							--	ENABLE DQ_OUT TO SEND DATA
			rst <= '1';							--	ENABLE DS1620 CHIP
			if en = 1 then
				dqo <= ptc_w(wnd);
				wnd := wnd + 1;
			else
				dqo <= ptc(nd);					--	SEND PORTOCOL DATA SET DS1620
				nd := nd + 1;					--	NEXT BIT DATA PORTOCOL
			end if;
			n := n + 1;							--	NEXT CLOCK
		end if;
	end if;

end if;
end process;
end ;
